#include <PsSimulatedObject.h>
#include <PsvMechanismPartner.h>

#include <PsController.h>
#include <PsObjectDescriptor.h>

#include <PsvTranslationYRotationInputHandler.h>
#include <PsvYRotationInputHandler.h>
#include <PsvZRotationInputHandler.h>
#include <cmath>
using namespace std;

#include "Camille.h"

Camille::Camille(PsController& ctrl, const PsObjectDescriptor& objectDescriptor) :
    PsSimulatedObject(ctrl, objectDescriptor),
    PsvMechanismPartner(),
    theta(0.),
    positionOutput(addOutput<PsTranslationYRotation>("position")),
    queueOutput(addOutput<PsYRotation>("queue")),
    patteArDOutput(addOutput<PsZRotation>("patteArD")),
    patteArGOutput(addOutput<PsZRotation>("patteArG")),
    patteAvDOutput(addOutput<PsZRotation>("patteAvD")),
    patteAvGOutput(addOutput<PsZRotation>("patteAvG"))
{
    // initialise the position output
    positionOutput.set(PsTranslationYRotation(0.,0.,0.,0.));
    queueOutput.set(PsYRotation(0.));
    patteArDOutput.set(PsZRotation(0.));
    patteArGOutput.set(PsZRotation(0.));
    patteAvDOutput.set(PsZRotation(0.));
    patteAvGOutput.set(PsZRotation(0.));
    
    // and make it visualised
    visualiseOutput<PsvTranslationYRotationInputHandler>(positionOutput, "DCS_position");
    visualiseOutput<PsvYRotationInputHandler>(queueOutput, "DCS_queue");
    visualiseOutput<PsvZRotationInputHandler>(patteArDOutput, "DCS_patteArD");
    visualiseOutput<PsvZRotationInputHandler>(patteArGOutput, "DCS_patteArG");
    visualiseOutput<PsvZRotationInputHandler>(patteAvDOutput, "DCS_patteAvD");
    visualiseOutput<PsvZRotationInputHandler>(patteAvGOutput, "DCS_patteAvG");
}


Camille::~Camille(void)
{
}

void Camille::init()
{
 inv=0;
}

void Camille::compute()
{
    // increment the angle
    theta += 0.5;
    
    if (angle>10) inv = 1;
    if (angle<-10) inv = 0;
    if (inv == 1) angle -= 0.5;
    else angle += 0.5;

    

    // angle in radians
    double theta_rad = theta * M_PI / 180.;
    
    // update the position
    positionOutput.set(PsTranslationYRotation(4. * cos(theta_rad), 0., 4. * sin(theta_rad), -theta - 90)); 
    queueOutput.set(PsYRotation(angle));
    patteArDOutput.set(PsZRotation(angle));
    patteArGOutput.set(PsZRotation(angle));
    patteAvDOutput.set(PsZRotation(-angle));
    patteAvGOutput.set(PsZRotation(-angle));

}
