// the simulation tree loader
#include <SimpleOpenMASK3Loader.h>
#include <PsController.h>

#include <PsKernelObjectAbstractFactory.h>

#include <PsvInteractive3DVis.h>
#include <Camera.h>
#include <PsKeyboardNavigator.h>

#include <PsException.h>

#include "Camille.h"

//------------------------------------------------------------------------------------------------------------------------
int main( int argc, char* argv[] )
{

    // construction of a simulation tree for root
    PsObjectDescriptor* simTree;

    if (argc >= 2)
    {
	// the first argument to camille is the simulation tree file
	simTree = (new SimpleOpenMASK3Loader(argv[1]))->getRootObjectDescriptor() ;
    }
    else
    {
	exit(1);
    }

    // we create the simulation controller
    PsController* controller = new PsController ( *simTree, 0 ) ;

    // next come the managers for objects in the simulation tree

    // visualisation
    controller->addInstanceCreator( "PsvInteractive3DVis",
				   new PsSimpleSimulatedObjectCreator<PsvInteractive3DVis> ()) ;
    // camera
    controller->addInstanceCreator ("Camera",
				   new PsSimpleSimulatedObjectCreator<Camera> ()) ;
    // keyboard navigation
    controller->addInstanceCreator ("PsKeyboardNavigator", 
				   new PsSimpleSimulatedObjectCreator<PsKeyboardNavigator> ()) ;
    // and our own object: Camille
    controller->addInstanceCreator ("Camille",
				   new PsSimpleSimulatedObjectCreator<Camille> () );

    // finally we initialise and run the simulation
    try
    {
	controller->init();
	controller->run();
    }
    catch (PsException& e)
    {
	cerr << "Uncaught exception: " << e << endl;
    }

    // and we exit (using Performer's exit function, which frees the graphical resources)
    pfExit();
}
 
